/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************

//INCLUDES
#include "driverlib.h"
#include <msp430.h>
#include <DRVGlobalVariables.h>
#include <math.h>

//Settings
#define numSamplesToAverage 20          //Number of samples from the ADC to average before calculating the angle
#define timer0PeriodMS 500              //Number of milliseconds for timer0 period, range: 1 <= timer0PeriodMS <= 2000
#define sw1LongPressTime 2              //# of timer loops to trigger the long-press button, range: sw1LongPressTime >= 2
                                        //Button long press in milliseconds = timer0PeriodMS*sw1LongPressTime
//Definitions
#define numCalPointsMax 36              //Used to allocate the array for the calibration points, which has a max of 36
#define PI 3.14159265                   //Definition of PI, used to calculate angle

//Board pin-out definitions
#define MUX0_PORT     GPIO_PORT_P4
#define MUX0_PIN      GPIO_PIN0
#define MUX1_PORT     GPIO_PORT_P8
#define MUX1_PIN      GPIO_PIN0

#define LED1_PORT     GPIO_PORT_P8
#define LED1_PIN      GPIO_PIN3
#define LED2_PORT     GPIO_PORT_P8
#define LED2_PIN      GPIO_PIN2
#define LED3_PORT     GPIO_PORT_P4
#define LED3_PIN      GPIO_PIN1
#define LED4_PORT     GPIO_PORT_P8
#define LED4_PIN      GPIO_PIN1

#define S1_PORT     GPIO_PORT_P1
#define S1_PIN      GPIO_PIN4

#define CS_PORT     GPIO_PORT_P5
#define CS_PIN      GPIO_PIN0

//Function definitions
void CLKSetup(void);                        //Function that sets the DCO, MCLK, and SMCLK
void initTimer(void);                       //Function to initialize the timer
void setTimer_A_Parameters(void);           //Function to set the timer parameters
void LCDSetup(void);                        //Function to set registers for LCD switching Frequency and turn on charge pump caps
void LCDOUT(int);                           //Function used to display to the LCD
void calibration(void);                     //Function to control the stages of calibration
void setMuxAddr(void);                      //Function to choose which Hall device(s) to use via the MUX


//Global ADC variables
double ADCdataAmax = 0;                     //Holds the maximum value received from ADC A during calibration
double ADCdataBmax = 0;                     //Holds the maximum value received from ADC B during calibration
double ADCdataAmin = 0xFFF;                 //Holds the minimum value received from ADC A during calibration
double ADCdataBmin = 0xFFF;                 //Holds the minimum value received from ADC B during calibration

double angleCalVal[numCalPointsMax] = {0};  //Holds the calibration angles

//Calibration mode variables
#define numCalOpts 16                       //The number of calibration point options to show on the LCD
int angleCalOpts[numCalOpts] = {1,2,3,4,5,6,8,9,10,12,15,18,20,24,30,36}; //Holds the calibration point options (use only divisors of 360)
int numCalPoints = 1;                       //Gets set to the selected value from the angleCalOpts array
#define numCalPointsStartIndex 6            //The index of the angleCalOpts array to display first
int numCalPointsStart = 0;                  //Allows use of a temporary 0 degree offset to display the desired option first and change with rotation

int calMode = 1;                            //Start at 1, determines the current calibration mode
int dispMode = 1;                           //Sets the display mode for the LCD based on current calibration type

int MUXadr = 0;                             //Holds the current MUX selection option to control which set of Hall devices are used
int MUXflip = 0;                            //Used to flip the X and Y components for angle calculation based on Hall device position

int sw1Flag = 1;                            //Indicates when the button (switch 1) has been pressed
int sw1LongPress = 0;                       //Flag for when the button has been long-pressed
Timer_A_initUpModeParam Timer0_A_params = {0}; //Create timer instance used for long-press detection
int calCount = 0;                           //Counts how many times the timer has triggered since a button press (for long-press detection)
int numSensors = 1;                         //Used to choose how many sensors to use for angle measurement



void main(void)
{
    //ADC and angle variables
    uint16_t ADCRXmsbA = 0;                     //Receives the msb of ADC A from SPI
    uint16_t ADCRXlsbA = 0;                     //Receives the lsb of ADC A from SPI

    uint16_t ADCRXmsbB = 0;                     //Receives the msb of ADC B from SPI
    uint16_t ADCRXlsbB = 0;                     //Receives the lsb of ADC B from SPI

    int ADCdataA = 0;                           //Holds the combined received data from SPI for ADC A
    int ADCdataB = 0;                           //Holds the combined received data from SPI for ADC B

    double ADCdataAsum = 0;                     //Holds the sum of the ADC A data to be averaged
    double ADCdataBsum = 0;                     //Holds the sum of the ADC B data to be averaged

    double ADCdataAnorm = 0;                    //Holds the normalized data for ADC A
    double ADCdataBnorm = 0;                    //Holds the normalized data for ADC B

    int index;                                  //The current index of angleCalVal to use for error correction
    int indexRef;                               //The reference index of angleCalVal to use for error correction
    double M;                                   //The slope value for error correction (from Y=MX+B)
    double B;                                   //The offset value for error correction (from Y=MX+B)

    double angleOffset = 0;                     //Holds the angle that is to be the 0 degree point

    double ADCangle = 0;                        //Holds the calculated angle before and after calibration
    int angleINT = 0;                           //Holds an integer version of the calculated angle to display on the LCD

    int i;                                      //Used in for loops throughout the program

    //Stop watchdog timer
    WDT_A_hold(WDT_A_BASE);

    // Configure SPI Pins for UCB0CLK, and UCB0RXD/UCB0SOMI
    //Select Port 5, set Pin 1, and Pin 3 to input with function.
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P5, GPIO_PIN1 + GPIO_PIN3, GPIO_PRIMARY_MODULE_FUNCTION);

    // Configure SPI Pins for UCA0CLK, and UCA0TXD/UCA0SIMO
    //Select Port 1, set Pin 0, and Pin 2 to input Secondary Module Function
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1, GPIO_PIN0 + GPIO_PIN2, GPIO_PRIMARY_MODULE_FUNCTION);

    //Disable the GPIO power-on default high-impedance mode to activate previously configured port settings
    PMM_unlockLPM5();

    //Initialize slave to MSB first, inactive high clock polarity and 3 wire SPI
    EUSCI_A_SPI_initSlaveParam paramA = {0};
    paramA.msbFirst = EUSCI_A_SPI_MSB_FIRST;
    paramA.clockPhase = EUSCI_A_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT;
    paramA.clockPolarity = EUSCI_A_SPI_CLOCKPOLARITY_INACTIVITY_HIGH;
    paramA.spiMode = EUSCI_A_SPI_3PIN;
    EUSCI_A_SPI_initSlave(EUSCI_A0_BASE, &paramA);

    //Initialize Master
    EUSCI_B_SPI_initMasterParam paramB = {0};
    paramB.selectClockSource = EUSCI_B_SPI_CLOCKSOURCE_SMCLK;
    paramB.clockSourceFrequency = CS_getSMCLK();
    paramB.desiredSpiClock = CS_getSMCLK();
    paramB.msbFirst = EUSCI_B_SPI_MSB_FIRST;
    paramB.clockPhase = EUSCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT;
    paramB.clockPolarity = EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_HIGH;
    paramB.spiMode = EUSCI_B_SPI_3PIN;
    EUSCI_B_SPI_initMaster(EUSCI_B0_BASE, &paramB);

    //Setup the Clock for the timers/board
    CLKSetup();

    //Enable SPI module
    EUSCI_A_SPI_enable(EUSCI_A0_BASE);
    EUSCI_B_SPI_enable(EUSCI_B0_BASE);

    //Configure Registers for LCD operation
    LCDSetup();

    //Prepare timer for counting
    initTimer();
    Timer0_A_params.timerPeriod = (((CS_getACLK()/1000)*timer0PeriodMS)-1);    // Set timer period, use timer0PeriodMS definition at top of program
    Timer_A_initUpMode(TIMER_A0_BASE, &Timer0_A_params);

    //Sets digital pins as output for LED's, MUX's, and the Control Signal
    GPIO_setAsOutputPin(CS_PORT,CS_PIN);
    GPIO_setAsOutputPin(MUX0_PORT,MUX0_PIN);
    GPIO_setAsOutputPin(MUX1_PORT,MUX1_PIN);
    GPIO_setAsOutputPin(LED1_PORT,LED1_PIN);
    GPIO_setAsOutputPin(LED2_PORT,LED2_PIN);
    GPIO_setAsOutputPin(LED3_PORT,LED3_PIN);
    GPIO_setAsOutputPin(LED4_PORT,LED4_PIN);

    //Sets digital pin as input for push button (Switch 1) and configures interrupt
    GPIO_setAsInputPin(S1_PORT,S1_PIN);
    GPIO_enableInterrupt(S1_PORT,S1_PIN);
    GPIO_selectInterruptEdge(S1_PORT,S1_PIN,GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_clearInterrupt(S1_PORT,S1_PIN);

    __bis_SR_register(GIE);      // enable interrupts

    while(1){

        //Calibrate at this stage of the code when the button has been pressed
        if(sw1Flag == 1){
            calibration();
        }

        //Get ADC samples to average
        for(i = 0; i < numSamplesToAverage; i++){

            //Set CS signal low to sample data for both ADC's
            GPIO_setOutputLowOnPin(CS_PORT, CS_PIN);

            //Transmit 0 byte to start SPI clock for reading data (MSB)
            EUSCI_A_SPI_transmitData(EUSCI_A0_BASE, 0);
            EUSCI_B_SPI_transmitData(EUSCI_B0_BASE, 0);

            //Get SPI data from RX buffer (MSB)
            if(MUXflip == 0){   //Use ADCA = X and ADCB = Y for math
                ADCRXmsbA = (EUSCI_A_SPI_receiveData(EUSCI_A0_BASE));//&0xFFF;
                ADCRXmsbB = (EUSCI_B_SPI_receiveData(EUSCI_B0_BASE));//&0xFFF;
            }else{              //Use ADCB = X and ADCA = Y for math
                ADCRXmsbB = (EUSCI_A_SPI_receiveData(EUSCI_A0_BASE));//&0xFFF;
                ADCRXmsbA = (EUSCI_B_SPI_receiveData(EUSCI_B0_BASE));//&0xFFF;
            }

            //Transmit 0 byte to continue SPI clock for reading data (LSB)
            EUSCI_A_SPI_transmitData(EUSCI_A0_BASE, 0);
            EUSCI_B_SPI_transmitData(EUSCI_B0_BASE, 0);

            //Get SPI data from RX buffer (LSB)
            if(MUXflip == 0){   //Use ADCA = X and ADCB = Y for math
                ADCRXlsbA = (EUSCI_A_SPI_receiveData(EUSCI_A0_BASE));
                ADCRXlsbB = (EUSCI_B_SPI_receiveData(EUSCI_B0_BASE));
            }else{              //Use ADCB = X and ADCA = Y for math
                ADCRXlsbB = (EUSCI_A_SPI_receiveData(EUSCI_A0_BASE));
                ADCRXlsbA = (EUSCI_B_SPI_receiveData(EUSCI_B0_BASE));
            }

            //Set CS signal high when done reading from ADC's
            GPIO_setOutputHighOnPin(CS_PORT, CS_PIN);

            //Combine MSB and LSB while removing unused bits
            ADCdataA = ((ADCRXmsbA & 0x7F) << 5) | ((ADCRXlsbA & 0xFF) >> 3);
            ADCdataB = ((ADCRXmsbB & 0x7F) << 5) | ((ADCRXlsbB & 0xFF) >> 3);

            //Add to sum for averaging
            ADCdataAsum += ADCdataA;
            ADCdataBsum += ADCdataB;

            //Get MIN and MAX values from ADC for calibration
            if(calMode == 2){
                if(ADCdataA > ADCdataAmax){
                    ADCdataAmax = ADCdataA;
                }

                if(ADCdataA < ADCdataAmin){
                    ADCdataAmin = ADCdataA;
                }

                if(ADCdataB > ADCdataBmax){
                    ADCdataBmax = ADCdataB;
                }

                if(ADCdataB < ADCdataBmin){
                    ADCdataBmin = ADCdataB;
                }
            }

        }

        //Get average
        ADCdataA = ADCdataAsum/numSamplesToAverage;
        ADCdataB = ADCdataBsum/numSamplesToAverage;

        //Normalize data to 1
        ADCdataAnorm = (ADCdataA-ADCdataAmin-(ADCdataAmax-ADCdataAmin)/2)/((ADCdataAmax-ADCdataAmin)/2);
        ADCdataBnorm = (ADCdataB-ADCdataBmin-(ADCdataBmax-ADCdataBmin)/2)/((ADCdataBmax-ADCdataBmin)/2);

        //Set values outside of 1 to be 1 as a railed value (to avoid error)
        if(ADCdataAnorm > 1){
            ADCdataAnorm = 1;
        }else if(ADCdataAnorm < -1){
            ADCdataAnorm = -1;
        }
        if(ADCdataBnorm > 1){
            ADCdataBnorm = 1;
        }else if(ADCdataBnorm < -1){
            ADCdataBnorm = -1;
        }

        //Get angle and convert to degrees
        if(numSensors == 2){    //Using 2 sensors, do atan2
            ADCangle = atan2(ADCdataBnorm,ADCdataAnorm)*180/PI;
        }else{                  //Using 1 sensor, do asin
            if(MUXadr < 2){
                ADCangle = 2*atan(ADCdataBnorm/(1+sqrt(1-ADCdataBnorm*ADCdataBnorm)))*180/PI;   //arcsine(x) = 2arctan(x/(1+sqrt(1-x^2))), used to conserve program space
            }else{
                ADCangle =2*atan(ADCdataAnorm/(1+sqrt(1-ADCdataAnorm*ADCdataAnorm)))*180/PI;    //arcsine(x) = 2arctan(x/1+sqrt(1-x^2))), used to conserve program space
            }
        }

        //Set degree range to 0-360 from atan2
        if(ADCangle < 0 && numSensors == 2){
            ADCangle += 360;
        }

        //Find real offset
        if(calMode == 4){
            angleOffset = ADCangle;
        }

        //Find temporary offset to use when selecting number of cal points
        if(calMode == 3 && numCalPointsStart == 0){
            angleOffset = ADCangle;
            numCalPointsStart = 1;
        }

        //Offset angle based on cal, and keep degree range at 0-360
        if(numSensors == 2){
            ADCangle -= angleOffset;
            if(ADCangle < 0){
                ADCangle += 360;
            }
        }

        //Get number of cal points
        if(calMode == 3){
            numCalPoints = (ADCangle/(360/numCalOpts))+numCalPointsStartIndex;  //Use current degree to to determine index for # of cal points selection
            numCalPoints = numCalPoints % numCalOpts;                           //Set range for index to numCalOpts
            numCalPoints = angleCalOpts[numCalPoints];                          //Get selected # of cal points from array
            LCDOUT(numCalPoints);                                               //Display currently selected # of cal points
        }

        //Get all calibration points
        if(calMode > 3){
            angleCalVal[calMode-4] = ADCangle;          //Store calibration sample point based on current calMode
        }

        //Calculate the error of the angle based on the error of two sample points using Y=MX+B for a linear error approximation correction
        if(calMode == 0 && numSensors == 2){
            index = 0;                                  //Catch for when the ADCangle is exactly 0
            for(i = 0; i < numCalPoints; i++){
                if(ADCangle > angleCalVal[i]){          //Find first calibration point to use
                    index = i;
                }
            }
            indexRef = (index+1) % numCalPoints;        //Get second calibration point to use
            if(indexRef == 0){                          //Find M and B when indexRef is 0
                M = (360/numCalPoints)/(360-angleCalVal[index]);
                B = ((360/numCalPoints)*index+(360/numCalPoints))-M*360;
            }else{                                      //Find M and B for all other cases
                M = (360/numCalPoints)/(angleCalVal[indexRef]-angleCalVal[index]);
                B = ((360/numCalPoints)*index+(360/numCalPoints))-M*angleCalVal[indexRef];
            }

            ADCangle = ADCangle*M+B;                    //Adjust for the estimated error calculated from Y=MX+B
        }

        angleINT = ADCangle*10 + 0.5;                   //Convert to int to use with LCD, *10 for one decimal precision, +0.5 and truncation for an easy round

        //Catch for rounding up to 360 degrees (display 0 instead)
        if(angleINT == 3600){
            angleINT = 0;
        }

        //Use negative sign on LCD when using negative numbers with one sensor (90 degrees)
        if(calMode == 0 && numSensors == 1){
            if(angleINT < 0){
                dispMode = 4;       //Display negative sign
                angleINT *= -1;     //Set number to positive to display
            }else{
                dispMode = 1;       //Use normal display mode (positive only)
            }
        }

        //Display the angle on the LCD when in normal operation.
        if(calMode == 0){
            LCDOUT(angleINT);
        }

        //Set to 0 for averaging
        ADCdataAsum = 0;
        ADCdataBsum = 0;
    }

}
void calibration(void){

    if(calMode == 1 && sw1LongPress == 1){  //Exit device selection mode and start calibration
        calMode++;
    }else if(sw1LongPress == 1){            //Restart calibration beginning with device selection
        //Preserve MUX selection
        if(MUXadr == 1){
            numSensors = (numSensors % 2) + 1;
        }
        MUXadr--;
        //Reset initial variable values for calibration
        dispMode = 1;
        calMode = 1;
        numCalPointsStart = 0;
        ADCdataAmax = 0;
        ADCdataBmax = 0;
        ADCdataAmin = 0xFFF;
        ADCdataBmin = 0xFFF;
    }else if(calMode > 1){                  //Increment calibration mode
        if(calMode == 2 && numSensors == 1){
            numCalPoints = 1;
            calMode += 2;                   //Skip calibration modes when using one sensor
        }
        calMode++;
    }else if(calMode == 0){                 //Do not calibrate (normal operation)
        return;
    }

    //CALIBRATE
    if(calMode == 1){                       //Select which Hall device configuration to use
        sel();
        MUXadr++;
        MUXadr = MUXadr % 4;
        if(MUXadr == 1){
            numSensors = (numSensors % 2) + 1; //Change between 1 and 2 sensor mode
        }
        setMuxAddr();
    }else if(calMode == 2){                 //Select number of cal points to use
        spin();
    }else if(calMode == 3){                 //Tell user to spin magnet to collect data
        dispMode = 2;
    }else if((calMode-3) <= numCalPoints){  //Display angle to calibrate to
        dispMode = 3;
        LCDOUT((calMode-4)*(360/numCalPoints));
    }else if((calMode-3) > numCalPoints){   //Check if calibration mode is done
        dispMode = 1;
        calMode = 0;
    }

    sw1Flag = 0;                            //Reset pushbutton flag
}

void setMuxAddr(void)
{
    int mux1 = (MUXadr & 2) >> 1;                       //Find MUX1 (MSB)
    int mux0 = MUXadr & 1;                              //Find MUX0 (LSB)

    GPIO_setOutputHighOnPin(LED1_PORT, LED1_PIN);       //LED1 OFF
    GPIO_setOutputHighOnPin(LED2_PORT, LED2_PIN);       //LED2 OFF
    GPIO_setOutputHighOnPin(LED3_PORT, LED3_PIN);       //LED3 OFF
    GPIO_setOutputHighOnPin(LED4_PORT, LED4_PIN);       //LED4 OFF

    if (mux1 == 1) {
        GPIO_setOutputHighOnPin(MUX1_PORT, MUX1_PIN);   //Use Hall device U2
        if(numSensors == 2 || (numSensors == 1 && MUXadr == 2)){
            GPIO_setOutputLowOnPin(LED2_PORT, LED2_PIN);    //LED2 ON
        }
    }
    else {
        GPIO_setOutputLowOnPin(MUX1_PORT, MUX1_PIN);    //Use Hall device U3
        if(numSensors == 2 || (numSensors == 1 && MUXadr == 1)){
            GPIO_setOutputLowOnPin(LED3_PORT, LED3_PIN);    //LED3 ON
        }
    }

    if (mux0 == 1) {
        GPIO_setOutputHighOnPin(MUX0_PORT, MUX0_PIN);   //Use Hall device U1
        if(numSensors == 2 || (numSensors == 1 && MUXadr == 3)){
            GPIO_setOutputLowOnPin(LED1_PORT, LED1_PIN);    //LED1 ON
        }
        MUXflip = 0;    //Use ADCA = X and ADCB = Y for math
    }
    else {
        GPIO_setOutputLowOnPin(MUX0_PORT, MUX0_PIN);    //Use Hall device U4
        if(numSensors == 2 || (numSensors == 1 && MUXadr == 0)){
            GPIO_setOutputLowOnPin(LED4_PORT, LED4_PIN);    //LED4 ON
        }
        MUXflip = 1;    //Use ADCB = X and ADCA = Y for math
    }
}


/*
 * ======== setTimer_A_Parameters ========
 */
// This function sets the timer A parameters
void setTimer_A_Parameters()
{
    Timer0_A_params.clockSource = TIMER_A_CLOCKSOURCE_ACLK;
    Timer0_A_params.clockSourceDivider = TIMER_A_CLOCKSOURCE_DIVIDER_1;
    Timer0_A_params.timerInterruptEnable_TAIE = TIMER_A_TAIE_INTERRUPT_DISABLE;
    Timer0_A_params.captureCompareInterruptEnable_CCR0_CCIE =
               TIMER_A_CAPTURECOMPARE_INTERRUPT_ENABLE;
    Timer0_A_params.timerClear = TIMER_A_DO_CLEAR;
    Timer0_A_params.startTimer = false;
}

/*
 * ======== initTimer ========
 */
void initTimer (void)
{

    setTimer_A_Parameters();                                //Configure timer parameters
    Timer_A_clearTimerInterrupt(TIMER_A0_BASE);             //Clear timer interrupt
    Timer0_A_params.timerPeriod = 0;                        //Set timer period to 0
    Timer_A_initUpMode(TIMER_A0_BASE, &Timer0_A_params);    //Configures Timer in up mode
}

/*
 * ======== TIMER0_A0_ISR ========
 */
#if defined(__TI_COMPILER_VERSION__) || (__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR (void)
#elif defined(__GNUC__) && (__MSP430__)
void __attribute__ ((interrupt(TIMER0_A0_VECTOR))) TIMER0_A0_ISR (void)
#else
#error Compiler not found!
#endif
{
    calCount++;                             //Count number of timer interrupts since button press

    if(calCount >= sw1LongPressTime){       //Check if long-press time has elapsed
        sw1LongPress = 1;                   //Set long press flag
        sw1Flag = 1;                        //Set button press flag
        Timer_A_stop(TIMER_A0_BASE);        //Stop timer
        calCount = 0;                       //Reset calCount
    }
}

//******************************************************************************
//
//This is the PORT1_VECTOR interrupt vector service routine
//
//******************************************************************************
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=PORT1_VECTOR
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(PORT1_VECTOR)))
#endif
void P1_ISR (void)
{

    if(GPIO_getInputPinValue(S1_PORT,S1_PIN) == 0){                             //Button pressed down
        calCount = 0;                                                           //Reset timer loop count
        Timer_A_startCounter(TIMER_A0_BASE,TIMER_A_UP_MODE);                    //Start button long-press timer
        GPIO_selectInterruptEdge(S1_PORT,S1_PIN,GPIO_LOW_TO_HIGH_TRANSITION);   //Look for button lifted up
    }else{                                                                      //Button lifted up
        Timer_A_stop(TIMER_A0_BASE);                                            //Stop button long-press timer
        if(sw1LongPress == 0){
            sw1Flag = 1;                                                        //Accept button press
        }
        GPIO_selectInterruptEdge(S1_PORT,S1_PIN,GPIO_HIGH_TO_LOW_TRANSITION);   //Look for button pressed down
    }

    sw1LongPress = 0;                                                           //Clear button long-press flag


    GPIO_clearInterrupt(S1_PORT,S1_PIN);                                        //S1 IFG cleared

}
